/**
 * 
 */
package net.bigeon.gclc.swt;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.swt.widgets.Label;

/** The runnable forwarding prompts from a buffered input to a label.
 * 
 * @author Emmanuel Bigeon */
public class PromptReadingRunnable implements Runnable {

    /** The logger for this class. */
    private static final Logger LOGGER = Logger
            .getLogger(PromptReadingRunnable.class.getName());

    /** The reader providing the succession of prompts */
    private final BufferedReader reader;
    /** The label. */
    private final Label          view;
    /** The running status. */
    private boolean              running = true;

    /** The updating runnable implementation.
     * 
     * @author Emmanuel Bigeon */
    private static class LabelUpdater implements Runnable {
        /** The label to update. */
        private final Label  label;
        /** The text to set. */
        private final String content;

        /** Create the updating runnable.
         * 
         * @param label the label
         * @param content the content */
        public LabelUpdater(final Label label, final String content) {
            super();
            this.label = label;
            this.content = content;
        }

        /* (non-Javadoc)
         * @see java.lang.Runnable#run() */
        @Override
        public void run() {
            label.setText(content);
        }
    }

    /** Create the prompt flow updating runnable.
     * 
     * @param reader the reader
     * @param view the view to update on lines. */
    public PromptReadingRunnable(final BufferedReader reader, final Label view) {
        super();
        this.reader = reader;
        this.view = view;
    }

    /* (non-Javadoc)
     * @see java.lang.Runnable#run() */
    @Override
    public void run() {
        String prompt;
        try {
            while (isRunning() && (prompt = reader.readLine()) != null) {
                view.getDisplay().syncExec(new LabelUpdater(view, prompt));
            }
        } catch (final IOException e) {
            LOGGER.log(Level.INFO, "Prompt cannot be read, or was prematuraly closed", e);
        }
    }

    /** Get the running state.
     * 
     * @return the running */
    public synchronized boolean isRunning() {
        return running;
    }

    /** Set the running state.
     * 
     * @param running the running to set */
    public synchronized void setRunning(final boolean running) {
        this.running = running;
    }

    /** Get the prompt flow.
     * 
     * @return the reader */
    public BufferedReader getReader() {
        return reader;
    }
}
