/**
 * 
 */
package net.bigeon.gclc.swt;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.doAnswer;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.io.IOException;

import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;
import org.junit.Test;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;

import net.bigeon.gclc.utils.PipedConsoleOutput;

/** @author Emmanuel Bigeon */
public class ConsoleOutputManagerTest {
    private final Text                 text    = mock(Text.class);
    private final ConsoleOutputManager com     = new ConsoleOutputManager(text);
    private final Display              display = mock(Display.class);
    {
        when(text.getDisplay()).thenReturn(display);
        doAnswer(new Answer<Object>() {
            /* (non-Javadoc)
             * @see
             * org.mockito.stubbing.Answer#answer(org.mockito.invocation.InvocationOnMock) */
            @Override
            public Object answer(final InvocationOnMock invocation) throws Throwable {
                final Runnable runnable = invocation.getArgument(0);
                runnable.run();
                return null;
            }
        }).when(display).syncExec(any(Runnable.class));
    }

    /** Test method for
     * {@link net.bigeon.gclc.swt.ConsoleOutputManager#appendLine(java.lang.String)}. */
    @Test
    public void testAppendConsoleOutput() {
        when(text.getText()).thenReturn("", "abc",
                "abc" + System.lineSeparator() + "def");
        com.appendLine("abc");
        verify(text).append("abc");
        com.appendLine("def");
        verify(text).append(System.lineSeparator());
        verify(text).append("def");
    }

    @Test
    public void testSetManager() throws IOException, InterruptedException {
        final PipedConsoleOutput output = mock(PipedConsoleOutput.class);
        com.setManager(output);
        com.setManager(output);

        try (PipedConsoleOutput output2 = new PipedConsoleOutput()) {
            com.setManager(output2);
            output2.println("line");
            final Thread forwardThread = com.getForwardThread();
            forwardThread.join(500);
            com.setManager(null);
            forwardThread.join();
            verify(text).append("line");
        }
    }
}
