/**
 * 
 */
package net.bigeon.gclc.swt;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.doAnswer;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.junit.Test;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;

/** @author Emmanuel Bigeon */
public class PromptReadingRunnableTest {
    private final Label   view    = mock(Label.class);
    private final Display display = mock(Display.class);
    {
        when(view.getDisplay()).thenReturn(display);
        doAnswer(new Answer<Object>() {
            /* (non-Javadoc)
             * @see
             * org.mockito.stubbing.Answer#answer(org.mockito.invocation.InvocationOnMock) */
            @Override
            public Object answer(final InvocationOnMock invocation) throws Throwable {
                final Runnable runnable = invocation.getArgument(0);
                runnable.run();
                return null;
            }
        }).when(display).syncExec(any(Runnable.class));
    }

    /** Test method for {@link net.bigeon.gclc.swt.PromptReadingRunnable#run()}.
     * 
     * @throws IOException if an error occurred */
    @Test
    public void testRun() throws IOException {
        final BufferedReader reader = mock(BufferedReader.class);
        when(reader.readLine()).thenReturn("abc", "def", null);

        final PromptReadingRunnable runnable = new PromptReadingRunnable(reader, view);

        runnable.run();
    }

    /** Test method for {@link net.bigeon.gclc.swt.PromptReadingRunnable#run()}.
     * 
     * @throws IOException if an error occurred */
    @Test
    public void testRunWithIO() throws IOException {
        final BufferedReader reader = mock(BufferedReader.class);
        when(reader.readLine()).thenThrow(new IOException());

        final PromptReadingRunnable runnable = new PromptReadingRunnable(reader, view);
        final Logger logger = Logger.getLogger(PromptReadingRunnable.class.getName());
        final Level back = logger.getLevel();
        logger.setLevel(Level.OFF);

        runnable.run();
        logger.setLevel(back);
    }

}
